<?php
namespace AkTest\Gdpr\Repository;

use Ak\Gdpr\Entity\GdprConsent;
use Ak\Gdpr\Repository\GdprConsentRepository;
use Ak\Gdpr\TableGateway\GdprConsentTable;
use Ak\Orm\Repository\RepositoryInterface;
use Ak\Orm\Repository\RepositoryManager;
use Ak\Test\AbstractTestCase;
use Ak\Test\DbTestTrait;
use AkTest\Gdpr\TestConfigProvider;
use Doctrine\DBAL\Schema\Schema;
use PHPUnit\DbUnit\DataSet\ArrayDataSet;
use PHPUnit\Framework\MockObject\MockObject;
use Zend\Db\Adapter\AdapterInterface;
use Zend\Hydrator\HydratorPluginManager;

/**
 * Class GdprConsentRepositoryTest
 *
 * @package AkTest\Gdpr\Repository
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/gdpr
 */
class GdprConsentRepositoryTest extends AbstractTestCase
{
    use DbTestTrait;

    public $configProviders = [
        TestConfigProvider::class,
    ];

    /**
     * @param Schema $schema
     * @throws \Doctrine\DBAL\Schema\SchemaException
     */
    public function createSchema(Schema $schema): void
    {
        if (!$schema->hasTable('gdpr_privole')) {
            $table = $schema->createTable('gdpr_privole');
            /** @var AdapterInterface|MockObject $adapter */
            $adapter = $this->createMock(AdapterInterface::class);
            $gateway = new GdprConsentTable('gdpr_privole', $adapter);
            $gateway->createSchema($table);
        }
    }

    protected function getDataSet()
    {
        return new ArrayDataSet([
            'gdpr_privole' => [
                ['id' => 1, 'email' => 'danijel@ep.hr', 'ip' => '192.168.0.34', 'time' => '2018-10-30 22:24:00']
            ],
        ]);
    }

    public function getRepositories(): RepositoryManager
    {
        $container = $this->getContainer();
        return $container->get(RepositoryManager::class);
    }

    public function getRepository(string $name): RepositoryInterface
    {
        $manager = $this->getRepositories();
        return $manager->get($name);
    }

    public function testIsRegistered()
    {
        $repos = $this->getRepositories();

        $this->assertTrue($repos->has(GdprConsent::class));
        $this->assertInstanceOf(GdprConsentRepository::class, $repos->get(GdprConsent::class));
    }

    public function testCanCreate()
    {
        $container = $this->lastContainer;
        $hydrators = $container->get(HydratorPluginManager::class);

        $consent = new GdprConsent();
        $consent->setEmail('user@domain.tld');
        $consent->setIp('1.2.3.4');

        $repository = new GdprConsentRepository($container->get('db.table.gdpr'), $hydrators->get(GdprConsent::class));
        /** @var GdprConsent $saved */
        $saved = $repository->insert($consent);

        $this->assertEquals(2, $saved->getId());
        $this->assertEquals('user@domain.tld', $saved->getEmail());
        $this->assertEquals('1.2.3.4', $saved->getIp());
        /** @noinspection PhpParamsInspection */
        $this->assertInstanceOf(\DateTime::class, $saved->getTime());

        $conn = $this->getConnection()->getConnection();
        $statement = $conn->prepare('SELECT * FROM gdpr_privole WHERE id = ?');
        $statement->execute([$saved->getId()]);
        $rows = $statement->fetchAll();

        $this->assertCount(1, $rows);
        $this->assertEquals(2, $rows[0]['id']);
        $this->assertEquals('user@domain.tld', $rows[0]['email']);
        $this->assertEquals('1.2.3.4', $rows[0]['ip']);
        $this->assertContains(date('Y-m-d H:i'), $rows[0]['time']);
    }

    public function testCanRead()
    {
        $container = $this->getContainer();
        $table = $container->get('db.table.gdpr');

        $hydrators = $container->get(HydratorPluginManager::class);
        $hydrator = $hydrators->get(GdprConsent::class);

        $repository = new GdprConsentRepository($table, $hydrator);

        $consent = $repository->find(1);
        $this->assertEquals(1, $consent->getId());
        $this->assertEquals('danijel@ep.hr', $consent->getEmail());
        $this->assertEquals('192.168.0.34', $consent->getIp());
        $this->assertInstanceOf(\DateTime::class, $consent->getTime());
        $this->assertEquals('2018-10-30 22:24:00', $consent->getTime()->format('Y-m-d H:i:s'));
    }

    public function testCanFindByEmail()
    {
        $container = $this->getContainer();
        $table = $container->get('db.table.gdpr');

        $hydrators = $container->get(HydratorPluginManager::class);
        $hydrator = $hydrators->get(GdprConsent::class);

        $repository = new GdprConsentRepository($table, $hydrator);

        $consent = $repository->findByEmail('danijel@ep.hr');
        $this->assertEquals(1, $consent->getId());
        $this->assertEquals('danijel@ep.hr', $consent->getEmail());
        $this->assertEquals('192.168.0.34', $consent->getIp());
        $this->assertInstanceOf(\DateTime::class, $consent->getTime());
        $this->assertEquals('2018-10-30 22:24:00', $consent->getTime()->format('Y-m-d H:i:s'));
    }

    public function testWillReturnNullForNotFound()
    {
        $repo = $this->getRepository(GdprConsent::class);
        $entity = $repo->find(123);

        $this->assertNull($entity);
    }

    /**
     * @expectedException \DomainException
     */
    public function testUpdateThrowsException()
    {
        $repo = $this->getRepository(GdprConsent::class);
        $repo->update(new GdprConsent());
    }

    /**
     * @expectedException \DomainException
     */
    public function testDeleteThrowsException()
    {
        $repo = $this->getRepository(GdprConsent::class);
        $repo->delete(new GdprConsent());
    }
}
